@@ -89,6 +89,7 @@ command_module_find_and_remove(COMMAND_REC *rec, SIGNAL_FUNC func)
 			if (cb->func == func) {
 				rec->callbacks =
 					g_slist_remove(rec->callbacks, cb);
+				g_free(cb);
 				return rec;
 			}
 		}
@@ -944,12 +945,18 @@ static void event_command(const char *line, SERVER_REC *server, void *item)
 	parse_command(line, expand_aliases, server, item);
 }
 
+static int eval_recursion_depth=0;
 /* SYNTAX: EVAL <command(s)> */
 static void cmd_eval(const char *data, SERVER_REC *server, void *item)
 {
 	g_return_if_fail(data != NULL);
+	if (eval_recursion_depth > 100) 
+		cmd_return_error(CMDERR_EVAL_MAX_RECURSE);
 
+ 
+	eval_recursion_depth++;
 	eval_special_string(data, "", server, item);
+	eval_recursion_depth--;
 }
 
 /* SYNTAX: CD <directory> */
