@@ -77,6 +77,14 @@ static void last_msg_dec_owns(GSList *list)
 	}
 }
 
+static void last_msg_destroy(GSList **list, LAST_MSG_REC *rec)
+{
+	*list = g_slist_remove(*list, rec);
+
+	g_free(rec->nick);
+	g_free(rec);
+}
+
 static void last_msg_add(GSList **list, const char *nick, int own, int max)
 {
 	LAST_MSG_REC *rec;
@@ -96,9 +104,8 @@ static void last_msg_add(GSList **list, const char *nick, int own, int max)
 		rec = g_new(LAST_MSG_REC, 1);
 		rec->nick = g_strdup(nick);
 
-		if ((int)g_slist_length(*list) == max) {
-			*list = g_slist_remove(*list,
-					       g_slist_last(*list)->data);
+		while ((int)g_slist_length(*list) >= max) {
+			last_msg_destroy(list, g_slist_last(*list)->data);
 		}
 
 		rec->own = own ? max : 0;
@@ -110,14 +117,6 @@ static void last_msg_add(GSList **list, const char *nick, int own, int max)
 	*list = g_slist_prepend(*list, rec);
 }
 
-static void last_msg_destroy(GSList **list, LAST_MSG_REC *rec)
-{
-	*list = g_slist_remove(*list, rec);
-
-	g_free(rec->nick);
-	g_free(rec);
-}
-
 void completion_last_message_add(const char *nick)
 {
 	g_return_if_fail(nick != NULL);
@@ -219,7 +218,6 @@ static void sig_message_own_private(SERVER_REC *server, const char *msg,
 				    const char *target, const char *origtarget)
 {
 	g_return_if_fail(server != NULL);
-	g_return_if_fail(target != NULL);
 
 	if (target != NULL && query_find(server, target) == NULL)
 		SERVER_LAST_MSG_ADD(server, target);
@@ -581,14 +579,11 @@ GList *completion_get_aliases(const char *word)
 }
 
 static void complete_window_nicks(GList **list, WINDOW_REC *window,
-                                  const char *word, const char *linestart)
+                                  const char *word, const char *nicksuffix)
 {
         CHANNEL_REC *channel;
         GList *tmplist;
         GSList *tmp;
-        const char *nicksuffix;
-
-        nicksuffix = *linestart != '\0' ? NULL : completion_char;
 
         channel = CHANNEL(window->active);
 
@@ -665,7 +660,8 @@ static void sig_complete_word(GList **list, WINDOW_REC *window,
 	} else if (channel != NULL) {
 		/* nick completion .. we could also be completing a nick
 		   after /MSG from nicks in channel */
-		complete_window_nicks(list, window, word, linestart);
+		const char *suffix = *linestart != '\0' ? NULL : completion_char;
+		complete_window_nicks(list, window, word, suffix);
 	} else if (window->level & MSGLEVEL_MSGS) {
 		/* msgs window, complete /MSG nicks */
                 *list = g_list_concat(completion_msg(server, NULL, word, NULL), *list);
@@ -709,6 +705,8 @@ static void sig_complete_msg(GList **list, WINDOW_REC *window,
 
 	msgserver = line_get_server(line);
 	*list = completion_msg(server, msgserver, word, NULL);
+	if (CHANNEL(window->active) != NULL)
+		complete_window_nicks(list, window, word, NULL);
 	if (*list != NULL) signal_stop();
 }
 
@@ -789,6 +787,36 @@ GList *completion_get_servers(const char *word)
 	return list;
 }
 
+GList *completion_get_targets(const char *word)
+{
+	CONFIG_NODE *node;
+	GList *list;
+	GSList *tmp;
+	int len;
+
+	g_return_val_if_fail(word != NULL, NULL);
+
+	len = strlen(word);
+	list = NULL;
+
+	/* get the list of all conversion targets */
+	node = iconfig_node_traverse("conversions", FALSE);
+	tmp = node == NULL ? NULL : config_node_first(node->value);
+	for (; tmp != NULL; tmp = config_node_next(tmp)) {
+		node = tmp->data;
+
+		if (node->type != NODE_TYPE_KEY)
+			continue;
+
+		if (len != 0 && g_strncasecmp(node->key, word, len) != 0)
+			continue;
+
+		list = g_list_append(list, g_strdup(node->key));
+	}
+	
+	return list;
+}
+
 static void sig_complete_connect(GList **list, WINDOW_REC *window,
 				 const char *word, const char *line, 
 				 int *want_space)
@@ -882,6 +910,32 @@ static void sig_complete_alias(GList **list, WINDOW_REC *window,
 	}
 }
 
+static void sig_complete_window(GList **list, WINDOW_REC *window,
+				const char *word, const char *linestart,
+				int *want_space)
+{
+	WINDOW_REC *win;
+	WI_ITEM_REC *item;
+	GSList *tmp;
+	int len;
+
+	g_return_if_fail(list != NULL);
+	g_return_if_fail(word != NULL);
+
+	len = strlen(word);
+
+	for (tmp = windows; tmp != NULL; tmp = tmp->next) {
+		win = tmp->data;
+		item = win->active;
+
+		if (win->name != NULL && g_strncasecmp(win->name, word, len) == 0)
+			*list = g_list_append(*list, g_strdup(win->name));
+		if (item != NULL && g_strncasecmp(item->visible_name, word, len) == 0)
+			*list = g_list_append(*list, g_strdup(item->visible_name));
+	}
+
+	if (*list != NULL) signal_stop();
+}
 
 static void sig_complete_channel(GList **list, WINDOW_REC *window,
 				 const char *word, const char *line,
@@ -905,6 +959,27 @@ static void sig_complete_server(GList **list, WINDOW_REC *window,
 	if (*list != NULL) signal_stop();
 }
 
+static void sig_complete_target(GList **list, WINDOW_REC *window,
+				const char *word, const char *line,
+				int *want_space)
+{
+	const char *definition;
+	
+	g_return_if_fail(list != NULL);
+	g_return_if_fail(word != NULL);
+	g_return_if_fail(line != NULL);
+
+	if (*line != '\0') {
+		if ((definition = iconfig_get_str("conversions", line ,NULL)) != NULL) {
+			*list = g_list_append(NULL, g_strdup(definition));
+			signal_stop();
+		}
+	} else {	
+		*list = completion_get_targets(word);
+		if (*list != NULL) signal_stop();
+	}
+}
+
 /* expand \n, \t and \\ */
 static char *expand_escapes(const char *line, SERVER_REC *server,
 			    WI_ITEM_REC *item)
@@ -1088,9 +1163,11 @@ void chat_completion_init(void)
 	signal_add("complete command away", (SIGNAL_FUNC) sig_complete_away);
 	signal_add("complete command unalias", (SIGNAL_FUNC) sig_complete_unalias);
 	signal_add("complete command alias", (SIGNAL_FUNC) sig_complete_alias);
+	signal_add("complete command window goto", (SIGNAL_FUNC) sig_complete_window);
 	signal_add("complete command window item move", (SIGNAL_FUNC) sig_complete_channel);
 	signal_add("complete command server add", (SIGNAL_FUNC) sig_complete_server);
 	signal_add("complete command server remove", (SIGNAL_FUNC) sig_complete_server);
+	signal_add("complete command recode remove", (SIGNAL_FUNC) sig_complete_target);
 	signal_add("message public", (SIGNAL_FUNC) sig_message_public);
 	signal_add("message join", (SIGNAL_FUNC) sig_message_join);
 	signal_add("message private", (SIGNAL_FUNC) sig_message_private);
@@ -1124,9 +1201,11 @@ void chat_completion_deinit(void)
 	signal_remove("complete command away", (SIGNAL_FUNC) sig_complete_away);
 	signal_remove("complete command unalias", (SIGNAL_FUNC) sig_complete_unalias);
 	signal_remove("complete command alias", (SIGNAL_FUNC) sig_complete_alias);
+	signal_remove("complete command window goto", (SIGNAL_FUNC) sig_complete_window);
 	signal_remove("complete command window item move", (SIGNAL_FUNC) sig_complete_channel);
 	signal_remove("complete command server add", (SIGNAL_FUNC) sig_complete_server);
 	signal_remove("complete command server remove", (SIGNAL_FUNC) sig_complete_server);
+	signal_remove("complete command recode remove", (SIGNAL_FUNC) sig_complete_target);
 	signal_remove("message public", (SIGNAL_FUNC) sig_message_public);
 	signal_remove("message join", (SIGNAL_FUNC) sig_message_join);
 	signal_remove("message private", (SIGNAL_FUNC) sig_message_private);
