@@ -127,8 +127,10 @@ static int get_scroll_count(void)
 
 	str = settings_get_str("scroll_page_count");
 	count = atof(str + (*str == '/'));
-	if (count <= 0)
+	if (count == 0)
 		count = 1;
+	else if (count < 0)
+		count = active_mainwin->height-active_mainwin->statusbar_lines+count;
 	else if (count < 1)
                 count = 1.0/count;
 
@@ -778,9 +780,7 @@ static void key_completion(int erase)
 	char *text, *line;
 	int pos;
 
-	pos = gui_entry_get_pos(active_entry);
-
-        text = gui_entry_get_text(active_entry);
+        text = gui_entry_get_text_and_pos(active_entry, &pos);
 	line = word_complete(active_win, text, &pos, erase);
 	g_free(text);
 
@@ -806,9 +806,7 @@ static void key_check_replaces(void)
 	char *text, *line;
 	int pos;
 
-	pos = gui_entry_get_pos(active_entry);
-
-        text = gui_entry_get_text(active_entry);
+        text = gui_entry_get_text_and_pos(active_entry, &pos);
 	line = auto_word_complete(text, &pos);
 	g_free(text);
 
@@ -1082,6 +1080,8 @@ void gui_readline_init(void)
 	key_bind("key", NULL, "meta2-c", "cright", (SIGNAL_FUNC) key_combo);
 	key_bind("key", NULL, "meta2-5D", "cleft", (SIGNAL_FUNC) key_combo);
 	key_bind("key", NULL, "meta2-5C", "cright", (SIGNAL_FUNC) key_combo);
+	key_bind("key", NULL, "meta2-1;5D", "cleft", (SIGNAL_FUNC) key_combo);
+	key_bind("key", NULL, "meta2-1;5C", "cright", (SIGNAL_FUNC) key_combo);
 
 	/* cursor movement */
 	key_bind("backward_character", "", "left", NULL, (SIGNAL_FUNC) key_backward_character);
