@@ -23,6 +23,9 @@
 #include "module.h"
 #include "textbuffer-view.h"
 #include "utf8.h"
+#ifdef HAVE_CUIX
+#include "cuix.h"
+#endif
 
 typedef struct {
 	char *name;
@@ -339,6 +342,7 @@ static int view_line_draw(TEXT_BUFFER_VIEW_REC *view, LINE_REC *line,
 	LINE_CACHE_REC *cache;
         const unsigned char *text, *end, *text_newline;
 	unsigned char *tmp;
+	unichar chr;
 	int xpos, color, drawcount, first, need_move, need_clrtoeol, char_width;
 
 	if (view->dirty) /* don't bother drawing anything - redraw is coming */
@@ -429,9 +433,9 @@ static int view_line_draw(TEXT_BUFFER_VIEW_REC *view, LINE_REC *line,
 			continue;
 		}
 
+		chr = *text;
 		end = text;
 		if (view->utf8) {
-			unichar chr;
 			if (get_utf8_char(&end, 6, &chr)<0)
 				char_width = 1;
 			else
@@ -447,15 +451,14 @@ static int view_line_draw(TEXT_BUFFER_VIEW_REC *view, LINE_REC *line,
 
 		xpos += char_width;
 		if (xpos <= term_width) {
-			if (*text >= 32 &&
-			    (end != text || (*text & 127) >= 32)) {
+			if ((chr & ~0x80) >= 32) {
 				for (; text < end; text++)
 					term_addch(view->window, *text);
 				term_addch(view->window, *text);
 			} else {
 				/* low-ascii */
 				term_set_color(view->window, ATTR_RESET|ATTR_REVERSE);
-				term_addch(view->window, (*text & 127)+'A'-1);
+				term_addch(view->window, (chr & 127)+'A'-1);
 				term_set_color(view->window, color);
 			}
 		}
@@ -1042,7 +1045,6 @@ static void bookmark_check_remove(char *key, LINE_REC *line,
 static void view_bookmarks_check(TEXT_BUFFER_VIEW_REC *view, LINE_REC *line)
 {
         BOOKMARK_FIND_REC rec;
-        LINE_REC *new_line;
 	GSList *tmp;
 
         rec.remove_line = line;
@@ -1051,16 +1053,9 @@ static void view_bookmarks_check(TEXT_BUFFER_VIEW_REC *view, LINE_REC *line)
 			     (GHFunc) bookmark_check_remove, &rec);
 
 	if (rec.remove_list != NULL) {
-		new_line = line->prev == NULL ? NULL :
-			(line->next == NULL ? line->prev : line->next);
 		for (tmp = rec.remove_list; tmp != NULL; tmp = tmp->next) {
 			g_hash_table_remove(view->bookmarks, tmp->data);
-			if (new_line != NULL) {
-				g_hash_table_insert(view->bookmarks,
-						    tmp->data, new_line);
-			} else {
-				g_free(tmp->data);
-			}
+			g_free(tmp->data);
 		}
 		g_slist_free(rec.remove_list);
 	}
@@ -1140,11 +1135,15 @@ static void view_remove_line(TEXT_BUFFER_VIEW_REC *view, LINE_REC *line,
 
 		if (view->startline == line) {
                         /* removing the first line in screen */
+			int is_last = view->startline->next == NULL;
+
 			realcount = view_scroll(view, &view->startline,
 						&view->subline,
 						linecount, FALSE);
 			view->ypos -= realcount;
 			view->empty_linecount += linecount-realcount;
+			if (is_last == 1)
+				view->startline = NULL;
 		}
 	} else {
 		if (textbuffer_line_exists_after(view->bottom_startline,
@@ -1194,6 +1193,24 @@ void textbuffer_view_remove_line(TEXT_BUFFER_VIEW_REC *view, LINE_REC *line)
 	textbuffer_remove(view->buffer, line);
 }
 
+void textbuffer_view_remove_lines_by_level(TEXT_BUFFER_VIEW_REC *view, int level)
+{
+	LINE_REC *line, *next;
+	
+	term_refresh_freeze();
+	line = textbuffer_view_get_lines(view);
+
+	while (line != NULL) {
+		next = line->next;
+
+		if (line->info.level & level)
+			textbuffer_view_remove_line(view, line);
+		line = next;
+	}
+	textbuffer_view_redraw(view);
+	term_refresh_thaw();
+}
+
 static int g_free_true(void *data)
 {
 	g_free(data);
@@ -1319,6 +1336,9 @@ static int sig_check_linecache(void)
 void textbuffer_view_init(void)
 {
 	linecache_tag = g_timeout_add(LINE_CACHE_CHECK_TIME, (GSourceFunc) sig_check_linecache, NULL);
+#ifdef HAVE_CUIX
+        cuix_active = 0;
+#endif
 }
 
 void textbuffer_view_deinit(void)
