@@ -20,6 +20,7 @@
 
 #include "module.h"
 #include "misc.h"
+#include "recode.h"
 #include "special-vars.h"
 #include "settings.h"
 #include "window-item-def.h"
@@ -63,6 +64,7 @@ static void cmd_notice(const char *data, IRC_SERVER_REC *server,
 		       WI_ITEM_REC *item)
 {
 	const char *target, *msg;
+	char *recoded;
 	void *free_arg;
 
         CMD_IRC_SERVER(server);
@@ -75,7 +77,10 @@ static void cmd_notice(const char *data, IRC_SERVER_REC *server,
 	if (*target == '\0' || *msg == '\0')
 		cmd_param_error(CMDERR_NOT_ENOUGH_PARAMS);
 
-	g_string_sprintf(tmpstr, "NOTICE %s :%s", target, msg);
+	recoded = recode_out(SERVER(server), msg, target);
+	g_string_sprintf(tmpstr, "NOTICE %s :%s", target, recoded);
+	g_free(recoded);
+
 	irc_send_cmd_split(server, tmpstr->str, 2, server->max_msgs_in_cmd);
 
 	cmd_params_free(free_arg);
@@ -102,8 +107,14 @@ static void cmd_ctcp(const char *data, IRC_SERVER_REC *server,
 	g_strup(ctcpcmd);
 	if (*ctcpdata == '\0')
 		g_string_sprintf(tmpstr, "PRIVMSG %s :\001%s\001", target, ctcpcmd);
-	else
-		g_string_sprintf(tmpstr, "PRIVMSG %s :\001%s %s\001", target, ctcpcmd, ctcpdata);
+	else {
+		char *recoded;
+
+		recoded = recode_out(SERVER(server), ctcpdata, target);
+		g_string_sprintf(tmpstr, "PRIVMSG %s :\001%s %s\001", target, ctcpcmd, recoded);
+		g_free(recoded);
+	}
+
 	irc_send_cmd_split(server, tmpstr->str, 2, server->max_msgs_in_cmd);
 
 	cmd_params_free(free_arg);
@@ -114,7 +125,7 @@ static void cmd_nctcp(const char *data, IRC_SERVER_REC *server,
 		      WI_ITEM_REC *item)
 {
 	const char *target;
-	char *ctcpcmd, *ctcpdata;
+	char *ctcpcmd, *ctcpdata, *recoded;
 	void *free_arg;
 
         CMD_IRC_SERVER(server);
@@ -128,7 +139,10 @@ static void cmd_nctcp(const char *data, IRC_SERVER_REC *server,
 		cmd_param_error(CMDERR_NOT_ENOUGH_PARAMS);
 
 	g_strup(ctcpcmd);
-	g_string_sprintf(tmpstr, "NOTICE %s :\001%s %s\001", target, ctcpcmd, ctcpdata);
+	recoded = recode_out(SERVER(server), ctcpdata, target);
+	g_string_sprintf(tmpstr, "NOTICE %s :\001%s %s\001", target, ctcpcmd, recoded);
+	g_free(recoded);
+
 	irc_send_cmd_split(server, tmpstr->str, 2, server->max_msgs_in_cmd);
 
 	cmd_params_free(free_arg);
@@ -139,6 +153,7 @@ static void cmd_part(const char *data, IRC_SERVER_REC *server,
 		     WI_ITEM_REC *item)
 {
 	char *channame, *msg;
+	char *recoded = NULL;
 	void *free_arg;
 
         CMD_IRC_SERVER(server);
@@ -153,16 +168,19 @@ static void cmd_part(const char *data, IRC_SERVER_REC *server,
         if (server->cmdcount > MAX_COMMANDS_ON_PART_UNTIL_PURGE)
 		irc_server_purge_output(server, channame);
 
-	irc_send_cmdv(server, *msg == '\0' ? "PART %s" : "PART %s :%s",
-		      channame, msg);
+	if (*msg != '\0')
+		recoded = recode_out(SERVER(server), msg, channame);
+	irc_send_cmdv(server, ! recoded ? "PART %s" : "PART %s :%s",
+		      channame, recoded);
 
+	g_free(recoded);
 	cmd_params_free(free_arg);
 }
 
 /* SYNTAX: KICK [<channel>] <nicks> [<reason>] */
 static void cmd_kick(const char *data, IRC_SERVER_REC *server, WI_ITEM_REC *item)
 {
-	char *channame, *nicks, *reason;
+	char *channame, *nicks, *reason, *recoded;
 	void *free_arg;
 
         CMD_IRC_SERVER(server);
@@ -175,7 +193,10 @@ static void cmd_kick(const char *data, IRC_SERVER_REC *server, WI_ITEM_REC *item
 	if (*channame == '\0' || *nicks == '\0') cmd_param_error(CMDERR_NOT_ENOUGH_PARAMS);
 	if (!ischannel(*channame)) cmd_param_error(CMDERR_NOT_JOINED);
 
-	g_string_sprintf(tmpstr, "KICK %s %s :%s", channame, nicks, reason);
+	recoded = recode_out(SERVER(server), reason, channame);
+	g_string_sprintf(tmpstr, "KICK %s %s :%s", channame, nicks, recoded);
+	g_free(recoded);
+
 	irc_send_cmd_split(server, tmpstr->str, 3, server->max_kicks_in_cmd);
 
 	cmd_params_free(free_arg);
@@ -186,6 +207,7 @@ static void cmd_topic(const char *data, IRC_SERVER_REC *server, WI_ITEM_REC *ite
 {
 	GHashTable *optlist;
 	char *channame, *topic;
+	char *recoded = NULL;
 	void *free_arg;
 
         CMD_IRC_SERVER(server);
@@ -195,9 +217,11 @@ static void cmd_topic(const char *data, IRC_SERVER_REC *server, WI_ITEM_REC *ite
 			    item, "topic", &optlist, &channame, &topic))
 		return;
 
-	irc_send_cmdv(server, *topic == '\0' &&
-		      g_hash_table_lookup(optlist, "delete") == NULL ?
-		      "TOPIC %s" : "TOPIC %s :%s", channame, topic);
+	if (*topic != '\0' || g_hash_table_lookup(optlist, "delete") != NULL)
+		recoded = recode_out(SERVER(server), topic, channame);
+	irc_send_cmdv(server, recoded == NULL ? "TOPIC %s" : "TOPIC %s :%s",
+		      channame, recoded);
+	g_free(recoded);
 
 	cmd_params_free(free_arg);
 }
@@ -345,7 +369,7 @@ static char *get_redirect_nicklist(const char *nicks, int *free)
 	return ret;
 }
 
-/* SYNTAX: WHOIS [<server>] [<nicks>] */
+/* SYNTAX: WHOIS [-<server tag>] [<server>] [<nicks>] */
 static void cmd_whois(const char *data, IRC_SERVER_REC *server,
 		      WI_ITEM_REC *item)
 {
@@ -356,10 +380,19 @@ static void cmd_whois(const char *data, IRC_SERVER_REC *server,
 
         CMD_IRC_SERVER(server);
 
-	if (!cmd_get_params(data, &free_arg, 2 | PARAM_FLAG_OPTIONS,
+	if (!cmd_get_params(data, &free_arg, 2 | PARAM_FLAG_OPTIONS |
+			    PARAM_FLAG_UNKNOWN_OPTIONS,
 			    "whois", &optlist, &qserver, &query))
 		return;
 
+	/* -<server tag> */
+	server = IRC_SERVER(cmd_options_get_server("whois", optlist,
+						   SERVER(server)));
+	if (server == NULL) {
+		cmd_params_free(free_arg);
+		return;
+	}
+
 	if (*query == '\0') {
 		query = qserver;
 		qserver = "";
@@ -382,28 +415,21 @@ static void cmd_whois(const char *data, IRC_SERVER_REC *server,
 	else {
 		g_string_sprintf(tmpstr, "WHOIS %s %s", qserver, query);
 		if (g_strcasecmp(qserver, query) == 0)
-			event_402 = "whois event noserver";
+			event_402 = "whois event not found";
 	}
 
 	query = get_redirect_nicklist(query, &free_nick);
 
 	str = g_strconcat(qserver, " ", query, NULL);
-	if (settings_get_bool("auto_whowas")) {
-		/* do automatic /WHOWAS if any of the nicks wasn't found */
-		server_redirect_event(server, "whois", 1, str, TRUE,
-				      NULL,
-				      "event 318", "whois end",
-				      "event 402", event_402,
-				      "event 301", "whois away", /* 301 can come as a reply to /MSG, /WHOIS or /WHOWAS */
-				      "event 401", "whois not found",
-				      "event 311", "whois event", NULL);
-	} else {
-		server_redirect_event(server, "whois", 1, str, TRUE,
-				      NULL,
-				      "event 318", "whois end",
-				      "event 301", "whois away", /* 301 can come as a reply to /MSG, /WHOIS or /WHOWAS */
-				      "event 311", "whois event", NULL);
-	}
+	server_redirect_event(server, "whois", 1, str, TRUE,
+		      NULL,
+		      "event 318", "whois end",
+		      "event 402", event_402,
+		      "event 301", "whois away", /* 301 can come as a reply to /MSG, /WHOIS or /WHOWAS */
+		      "event 313", "whois oper",
+		      "event 401", (settings_get_bool("auto_whowas") ? "whois try whowas" : "whois event not found"),
+		      "event 311", "whois event",
+		      "", "whois default event", NULL);
         g_free(str);
 
 	server->whois_found = FALSE;
@@ -420,7 +446,7 @@ static void event_whois(IRC_SERVER_REC *server, const char *data,
 	signal_emit("event 311", 4, server, data, nick, addr);
 }
 
-static void sig_whois_not_found(IRC_SERVER_REC *server, const char *data)
+static void sig_whois_try_whowas(IRC_SERVER_REC *server, const char *data)
 {
 	char *params, *nick;
 
@@ -452,16 +478,16 @@ static void event_whowas(IRC_SERVER_REC *server, const char *data,
 	signal_emit("event 314", 4, server, data, nick, addr);
 }
 
-/* SYNTAX: WHOWAS [<nicks> [<count>]] */
+/* SYNTAX: WHOWAS [<nicks> [<count> [server]]] */
 static void cmd_whowas(const char *data, IRC_SERVER_REC *server)
 {
-	char *nicks, *count, *nicks_redir;
+	char *nicks, *rest, *nicks_redir;
 	void *free_arg;
 	int free_nick;
 
         CMD_IRC_SERVER(server);
 
-	if (!cmd_get_params(data, &free_arg, 2, &nicks, &count))
+	if (!cmd_get_params(data, &free_arg, 2 | PARAM_FLAG_GETREST, &nicks, &rest))
 		return;
 	if (*nicks == '\0') nicks = server->nick;
 
@@ -472,8 +498,8 @@ static void cmd_whowas(const char *data, IRC_SERVER_REC *server)
 	if (free_nick) g_free(nicks_redir);
 
 	server->whowas_found = FALSE;
-	irc_send_cmdv(server, *count == '\0' ? "WHOWAS %s" :
-		      "WHOWAS %s %s", nicks, count);
+	irc_send_cmdv(server, *rest == '\0' ? "WHOWAS %s" :
+		      "WHOWAS %s %s", nicks, rest);
 
 	cmd_params_free(free_arg);
 }
@@ -551,7 +577,10 @@ static void cmd_quote(const char *data, IRC_SERVER_REC *server)
 	if (server == NULL || server->connect_time == 0)
 		cmd_return_error(CMDERR_NOT_CONNECTED);
 
-	irc_send_cmd(server, data);
+	if (!server->connected)
+		irc_send_cmd_now(server, data);
+	else
+		irc_send_cmd(server, data);
 }
 
 /* SYNTAX: RAWQUOTE <data> */
@@ -608,7 +637,7 @@ static void cmd_wait(const char *data, IRC_SERVER_REC *server)
 /* SYNTAX: WALL [<channel>] <message> */
 static void cmd_wall(const char *data, IRC_SERVER_REC *server, WI_ITEM_REC *item)
 {
-	char *channame, *msg, *args;
+	char *channame, *msg, *args, *recoded;
 	void *free_arg;
 	IRC_CHANNEL_REC *chanrec;
 	GSList *tmp, *nicks;
@@ -623,31 +652,46 @@ static void cmd_wall(const char *data, IRC_SERVER_REC *server, WI_ITEM_REC *item
 	chanrec = irc_channel_find(server, channame);
 	if (chanrec == NULL) cmd_param_error(CMDERR_CHAN_NOT_FOUND);
 
-	/* send notice to all ops */
-	nicks = NULL;
-	g_hash_table_foreach(chanrec->nicks, (GHFunc) cmd_wall_hash, &nicks);
+	/* See if the server has advertised support of wallchops */
+	if (g_hash_table_lookup(chanrec->server->isupport, "statusmsg") ||
+	    g_hash_table_lookup(chanrec->server->isupport, "wallchops"))
+		irc_send_cmdv(server, "NOTICE @%s :%s", chanrec->name, msg);
+	else {
+		/* Fall back to manually noticing each op */
+		nicks = NULL;
+		g_hash_table_foreach(chanrec->nicks,
+				     (GHFunc) cmd_wall_hash, &nicks);
 
-	args = g_strconcat(chanrec->name, " ", msg, NULL);
-	msg = parse_special_string(settings_get_str("wall_format"),
-				   SERVER(server), item, args, NULL, 0);
-	g_free(args);
+		args = g_strconcat(chanrec->name, " ", msg, NULL);
+		msg = parse_special_string(settings_get_str("wall_format"),
+					   SERVER(server), item, args, NULL, 0);
+		g_free(args);
 
-	for (tmp = nicks; tmp != NULL; tmp = tmp->next) {
-		NICK_REC *rec = tmp->data;
+		recoded = recode_out(SERVER(server), msg, channame);
 
-		if (rec != chanrec->ownnick)
-			irc_send_cmdv(server, "NOTICE %s :%s", rec->nick, msg);
+		for (tmp = nicks; tmp != NULL; tmp = tmp->next) {
+			NICK_REC *rec = tmp->data;
+
+			if (rec != chanrec->ownnick) {
+				irc_send_cmdv(server, "NOTICE %s :%s",
+					      rec->nick, recoded);
+			}
+		}
+
+		g_free(recoded);
+		g_free(msg);
+		g_slist_free(nicks);
 	}
-	g_free(msg);
-	g_slist_free(nicks);
 
 	cmd_params_free(free_arg);
 }
 
 /* SYNTAX: WALLCHOPS <channel> <message> */
+/* ircu is the only major server i can see which supports this 
+   and it supports NOTICE @#channel anyway */
 static void cmd_wallchops(const char *data, IRC_SERVER_REC *server, WI_ITEM_REC *item)
 {
-	char *channame, *msg;
+	char *channame, *msg, *recoded;
 	void *free_arg;
 
         CMD_IRC_SERVER(server);
@@ -657,8 +701,10 @@ static void cmd_wallchops(const char *data, IRC_SERVER_REC *server, WI_ITEM_REC
 		return;
 	if (*msg == '\0') cmd_param_error(CMDERR_NOT_ENOUGH_PARAMS);
 
-	irc_send_cmdv(server, "WALLCHOPS %s :%s", channame, msg);
+	recoded = recode_out(SERVER(server), msg, channame);
+	irc_send_cmdv(server, "WALLCHOPS %s :%s", channame, recoded);
 
+	g_free(recoded);
 	cmd_params_free(free_arg);
 }
 
@@ -667,7 +713,7 @@ static void cmd_kickban(const char *data, IRC_SERVER_REC *server,
 			WI_ITEM_REC *item)
 {
         IRC_CHANNEL_REC *chanrec;
-	char *channel, *nicks, *reason, *kickcmd, *bancmd;
+	char *channel, *nicks, *reason, *kickcmd, *bancmd, *recoded;
         char **nicklist, *spacenicks;
 	void *free_arg;
 
@@ -690,7 +736,10 @@ static void cmd_kickban(const char *data, IRC_SERVER_REC *server,
         spacenicks = g_strjoinv(" ", nicklist);
 	g_strfreev(nicklist);
 
-	kickcmd = g_strdup_printf("%s %s %s", chanrec->name, nicks, reason);
+	recoded = recode_out(SERVER(server), reason, channel);
+	kickcmd = g_strdup_printf("%s %s %s", chanrec->name, nicks, recoded);
+	g_free(recoded);
+
 	bancmd = g_strdup_printf("%s %s", chanrec->name, spacenicks);
         g_free(spacenicks);
 
@@ -749,7 +798,7 @@ static void cmd_knockout(const char *data, IRC_SERVER_REC *server,
 			 IRC_CHANNEL_REC *channel)
 {
 	KNOCKOUT_REC *rec;
-	char *nicks, *reason, *timeoutstr, *kickcmd, *bancmd;
+	char *nicks, *reason, *timeoutstr, *kickcmd, *bancmd, *recoded;
         char **nicklist, *spacenicks, *banmasks;
 	void *free_arg;
 	int timeleft;
@@ -785,7 +834,10 @@ static void cmd_knockout(const char *data, IRC_SERVER_REC *server,
 	banmasks = ban_get_masks(channel, spacenicks, 0);
 	g_free(spacenicks);
 
-	kickcmd = g_strdup_printf("%s %s %s", channel->name, nicks, reason);
+	recoded = recode_out(SERVER(server), reason, channel->name);
+	kickcmd = g_strdup_printf("%s %s %s", channel->name, nicks, recoded);
+	g_free(recoded);
+
 	bancmd = *banmasks == '\0'? NULL :
 		g_strdup_printf("%s %s", channel->name, banmasks);
 	
@@ -840,6 +892,8 @@ static void sig_server_disconnected(IRC_SERVER_REC *server)
 	if (!IS_IRC_SERVER(server))
 		return;
 
+	g_free(server->last_nick);
+
 	while (server->knockoutlist != NULL)
 		knockout_destroy(server, server->knockoutlist->data);
 }
@@ -957,6 +1011,8 @@ void irc_commands_init(void)
 	command_bind_irc("admin", NULL, (SIGNAL_FUNC) command_self);
 	/* SYNTAX: INFO [<server>] */
 	command_bind_irc("info", NULL, (SIGNAL_FUNC) command_self);
+    /* SYNTAX: KNOCK <channel> */
+    command_bind_irc("knock", NULL, (SIGNAL_FUNC) command_self);
 	/* SYNTAX: LINKS [[<server>] <mask>] */
 	command_bind_irc("links", NULL, (SIGNAL_FUNC) command_self);
 	/* SYNTAX: LUSERS [<server mask> [<remote server>]] */
@@ -1012,7 +1068,7 @@ void irc_commands_init(void)
 
 	signal_add("channel destroyed", (SIGNAL_FUNC) sig_channel_destroyed);
 	signal_add("server disconnected", (SIGNAL_FUNC) sig_server_disconnected);
-	signal_add("whois not found", (SIGNAL_FUNC) sig_whois_not_found);
+	signal_add("whois try whowas", (SIGNAL_FUNC) sig_whois_try_whowas);
 	signal_add("whois event", (SIGNAL_FUNC) event_whois);
 	signal_add("whois end", (SIGNAL_FUNC) event_end_of_whois);
 	signal_add("whowas event", (SIGNAL_FUNC) event_whowas);
@@ -1048,6 +1104,7 @@ void irc_commands_deinit(void)
 	command_unbind("ison", (SIGNAL_FUNC) command_1self);
 	command_unbind("admin", (SIGNAL_FUNC) command_self);
 	command_unbind("info", (SIGNAL_FUNC) command_self);
+    command_unbind("knock", (SIGNAL_FUNC) command_self);
 	command_unbind("links", (SIGNAL_FUNC) command_self);
 	command_unbind("lusers", (SIGNAL_FUNC) command_self);
 	command_unbind("map", (SIGNAL_FUNC) command_self);
@@ -1082,7 +1139,7 @@ void irc_commands_deinit(void)
 
 	signal_remove("channel destroyed", (SIGNAL_FUNC) sig_channel_destroyed);
 	signal_remove("server disconnected", (SIGNAL_FUNC) sig_server_disconnected);
-	signal_remove("whois not found", (SIGNAL_FUNC) sig_whois_not_found);
+	signal_remove("whois try whowas", (SIGNAL_FUNC) sig_whois_try_whowas);
 	signal_remove("whois event", (SIGNAL_FUNC) event_whois);
 	signal_remove("whois end", (SIGNAL_FUNC) event_end_of_whois);
 	signal_remove("whowas event", (SIGNAL_FUNC) event_whowas);
